/******************************************************************************		
*                                                                                   	
*       Copyright (C) 2006 Freescale Semiconductor, Inc.                            	
*       All Rights Reserved								              				
*														              					
* Filename:      bout.c               											
* 
* Author:        Christine Peng
*
* Release:       August 15, 2006            											
*														             													              					
* Description:	 Brownout System Protection Software for MC9S08DZ60.
*                This software utilizes the on-chip system protection
*                features of the MC9S08DZ60 for brownout protection.
*                Developed with CodeWarrior 5.0 	        	    	
*                       						
*																						
* Notes:         
*                COP Reset event -  PTD0 = 0  (LED ON)
*                ILOP Reset event - PTD1 = 0  (LED ON)
*                ILAD Reset event - PTD3 = 0  (LED ON)
*                MCG external clock loss of clock event - RESET MCU - PTD4 = 0 (LED ON)  
*                LVD event - RESET MCU - PTD5 = 0 (LED ON)
*                LVW event - PTD6 = 0 (LED ON)-> 1 (OFF) if Vdd rises above 4.7 V again
*                MCG external clock loss of lock event - PTD7 = 0 (LED ON)
*
*                To test MCG LOL and LOC, a 4 MHz crystal is required with
*                this software, and "#define EXTXTAL" in "bout.h" must be
*                uncommented. Otherwise, LOL and LOC will never occur because
*                the MCG will use the internal oscillator.   
* 																		
*  ***********************************************************************  			
*  *  THIS CODE IS ONLY INTENDED AS AN EXAMPLE OF CODE FOR THE           *  	 		
*  *  CODEWARRIOR COMPILER AND HAS ONLY BEEN GIVEN A MIMIMUM             *  	 		
*  *  LEVEL OF TEST. IT IS PROVIDED 'AS SEEN' WITH NO GUARANTEES         *  	 		
*  *  AND NO PROMISE OF SUPPORT.                                         *  	 		
*  ***********************************************************************  	 		
*																						
* Freescale reserves the right to make changes without further notice to any			
* product herein to improve reliability, function, or design. Freescale does	 		
* not assume any  liability arising  out  of the  application or use of any 	 		
* product,  circuit, or software described herein;  neither  does it convey 	 		
* any license under its patent rights  nor the  rights of others.  Freescale	 		
* products are not designed, intended,  or authorized for use as components 	 		
* in  systems  intended  for  surgical  implant  into  the  body, or  other 	 		
* applications intended to support life, or  for any  other application  in 	 		
* which the failure of the Freescale product  could create a situation where	 		
* personal injury or death may occur. Should Buyer purchase or use Freescale	 		
* products for any such intended  or unauthorized  application, Buyer shall 	 		
* indemnify and  hold  Freescale  and its officers, employees, subsidiaries,	 		
* affiliates,  and distributors harmless against all claims costs, damages, 	  		
* and expenses, and reasonable  attorney  fees arising  out of, directly or 	 		
* indirectly,  any claim of personal injury  or death  associated with such 	 		
* unintended or unauthorized use, even if such claim alleges that  Freescale	 		
* was negligent regarding the  design  or manufacture of the part. Freescale	 		
* and the Freescale logo* are registered trademarks of Freescale Ltd.       	 		
******************************************************************************/		


/* INCLUDE FILES */
#include <MC9S08DZ60.h>
#include "bout.h"

/* GLOBAL VARIABLE */
const byte Ill_Op_Fill[]    @0x00005F00 = {
    0xAC, 0xAC, 0xAC, 0xAC, 0xAC, 0xAC, 0xAC, 0xAC
};

/******************************************************************************
/ Function Name	:	PortsInit
/ Notes			    :	Initializes port pins to a known state as outputs driving low.
/                 This minimizes current consumption.
******************************************************************************/

void PortsInit(void){

  PTASE = 0x00;    /* Slew Rate Enable Registers */
  PTBSE = 0x00;
  PTCSE = 0x00;
  PTDSE = 0x00;
  PTESE = 0x00;
  PTFSE = 0x00;
  PTGSE = 0x00;

  PTADS = 0xFF;    /* Drive Strength Selection Registers */ 
  PTBDS = 0xFF;
  PTCDS = 0xFF;
  PTDDS = 0xFF;
  PTEDS = 0xFF;
  PTFDS = 0xFF;
  PTGDS = 0xFF;
 
  PTADD = 0xFF;    /* Data Direction Registers */
  PTBDD = 0xF0;    /* PTB 3-0 used as input switches */
  PTCDD = 0xFF;
  PTDDD = 0xFF;
  PTEDD = 0xFF;
  PTFDD = 0xFF;
  PTGDD = 0xFF;
  
  PTAD = 0x00;     /* Data Registers */
  PTBD = 0x00;
  PTCD = 0x00;
  PTDD = 0xFF;     /* ALL LEDs off (PTD pins set to 1) */
  PTED = 0x00;
  PTFD = 0x00;
  PTGD = 0x00;
}

/******************************************************************************
/ Function Name	:	MCGInit
/ Notes			    :	If 4 MHz crystal available, puts MCG into PLL engaged external
/                 mode running at 16 MHz bus. Otherwise leaves MCG in default FLL
/                 engaged internal mode running at 8 MHz bus.  If a loss of clock
/                 occurred previously bc some external event interfered with or 
/                 destroyed the crystal clock source, the MCG will be configured for 
/                 FLL engaged internal (FEI) mode running at 16 MHz bus.            
******************************************************************************/

void MCGInit (void) {
   if (ExtClkFlg == 1) { /* If using a 4 MHz crystal */
       if (SRS_LOC == 0)  {   /* If no loss of clock has occurred, enter PEE to generate 16 MHz bus */
           /* FEI -> FBE Mode */ 
           MCGC2 = 0x36;      /* BDIV by 1, xtal freq RANGE high, High GAIN, EREFS set */
           MCGC1 = 0xB8;      /* RDIV by 128 down to 31.25 kHz for FLL, Switch to ext. ref  */
           while (MCGSC_IREFST);       /* Check internal ref. no longer on */
           MCGC3 = 0xA0;      /* LOL interrupt, LOC reset (CME) enabled (ONLY after IREFS = 0)  */  
           while (!MCGSC_OSCINIT);     /* Wait until crystal initializes */
           while (MCGSC_CLKST != 0b10);/* Check external ref. switched */
       
           /* FBE -> PBE */ 
           MCGC3 = 0xE8;      /* Set PLLS bit with VDIV mult by 32 */  
           MCGC1 = 0x90;      /* RDIV by 4 (4 MHz/4 = 1 MHz for PLL) */ 
           while (!MCGSC_PLLST);
           while (!MCGSC_LOCK);  

           /* PBE -> PEE */
           MCGC1 =  0x10;     /* CLKS changed to [0:0] - PLL output = 32 MHz clock, 16 MHz bus */
           while (MCGSC_CLKST != 0b11);
       }

   }
   
   /* If internal clock desired, default FEI mode out of reset at 
      8 MHz bus (BDIV by 2) is used to clock MCU instead.
   */
}

/******************************************************************************
/ Function Name	:	PWMInit
/ Notes			    :	Generates a PWM signal on TPM1Ch0 (PTD0) at a frequency of
/                 busclk/10.  With an oscilloscope connected to the port pin,
/                 the user can monitor the bus frequency and ensure that it is
/                 in the proper frequency range.
******************************************************************************/

void PWMInit (void)  {
    TPM1MODH = 0x00;    /* PWM period = BUSCLK x 10 */
    TPM1MODL = 0x09;    /* TPM counter counts 0-9 (10 busclk counts */
    TPM1C0SC = 0x24;    /* Edge-aligned PWM */
    TPM1C0VH = 0x00;    /* 5/10 = 50% duty cycle */
    TPM1C0VL = 0x05;
    TPM1SC = 0x08;      /* TPM clock source is BUSCLK */
}

/******************************************************************************
/ Function Name	:	TestOptions
/ Notes			    :	Using switches on demo board (PTB pins), determines which 
/                 options to test.
******************************************************************************/
void TestOptions (void) {  
    if (PTBD_PTBD0==1) {  /* Turn COP off and test LVW interrupts - when testing LVW for this software only */
        SOPT1 = 0x00;     /* For test, manually decrease power supply voltage = too much time in LVW ISR -  */
        SPMSC1_LVWIE = 1; /* no time to clear COP in main loop results in continuous COP resets  */
    } else {
        SOPT1 = 0x40;     /* Turn COP on - 2^13 bus cycle time out period */
        SPMSC1_LVWIE = 0; /* Disallow LVW interrupts */
    }
                                                  
    if (PTBD_PTBD1 == 1) { /* Test ILOP reset */ 
        _asm JMP 0x5F00   /* jump to ILOP location */ 
    } 
    
    if (PTBD_PTBD2 == 1) {   
        ExtClkFlg = 1;  /* Use external 4 MHz crystal to test LOC reset*/
    }  else {
        ExtClkFlg = 0;  /* otherwise use default internal FEI mode  at 8 MHz bus freq */ 
    }
    if (PTBD_PTBD3 == 1) {
        SOPT2 = 0xC0;      /* Test COP reset - turn windowed COP on */  
        __RESET_WATCHDOG();/* Reset COP at inappropriate time - generates COP reset */
      
    }
                 
}              


/******************************************************************************
/ Function Name	:	ResetErrorHandler
/ Notes			    :	Handles resets due to various error conditions.
/                 
******************************************************************************/

void ResetErrorHandler (void) {
    if (SRS_COP==1) {
        PTDD_PTDD0 = 0;       /* PTD0 LED ON if Reset occurred due to COP timeout */
    }
    if (SRS_ILOP==1) {
        PTDD_PTDD1 = 0;       /* PTD1 LED ON if Reset occurred due to illegal op-code */
    }
    if (SRS_ILAD==1) {
        PTDD_PTDD3 = 0;       /* PTD3 if Reset occurred due to illegal address access */
    }
    if (SRS_LOC==1)  {        /* If reset due to LOC (SRS_LOC==1), switch to FEI (16 MHz bus) */  
        MCGC2 = 0x00;         /* BDIV is div by 1, FEI mode */
        MCGC1 = 0x04;         /* internal ref. selected */
        PTDD_PTDD4 = 0;       /* PTD4 LED ON if Reset occurred due to loss of external clock src */
    }
    if ((SRS_LVD==1)&&(SRS_POR!=1)) {
        PTDD_PTDD5 = 0;       /* PTD5 LED ON if Reset occurred due solely to LVD event */
    }
      
}