/******************************************************************************		
*                                                                                   	
*       Copyright (C) 2006 Freescale Semiconductor, Inc.                            	
*       All Rights Reserved								              				
*														              					
* Filename:      main.c  
* 
* Author:        Christine Peng
*
* Release:       August 15, 2006            											
*														              					
* Description:	 Brownout System Protection Software for MC9S08DZ60.
*                This software utilizes the on-chip system protection
*                features of the MC9S08DZ60 for brownout protection.
*                Developed with CodeWarrior 5.0 	        	    	
*                       						
*																						
* Notes:         
*                COP Reset event -  PTD0 = 0  (LED ON)
*                ILOP Reset event - PTD1 = 0  (LED ON)
*                ILAD Reset event - PTD3 = 0  (LED ON)
*                MCG external clock loss of clock event - RESET MCU - PTD4 = 0 (LED ON)  
*                LVD event - RESET MCU - PTD5 = 0 (LED ON)
*                LVW event - PTD6 = 0 (LED ON)-> 1 (OFF) if Vdd rises above 4.7 V again
*                MCG external clock loss of lock event - PTD7 = 0 (LED ON)
*
*                To test MCG LOL and LOC, a 4 MHz crystal is required with
*                this software, and "#define EXTXTAL" in "bout.h" must be
*                uncommented. Otherwise, LOL and LOC will never occur because
*                the MCG will use the internal oscillator.   
* 																		
*  ***********************************************************************  			
*  *  THIS CODE IS ONLY INTENDED AS AN EXAMPLE OF CODE FOR THE           *  	 		
*  *  CODEWARRIOR COMPILER AND HAS ONLY BEEN GIVEN A MIMIMUM             *  	 		
*  *  LEVEL OF TEST. IT IS PROVIDED 'AS SEEN' WITH NO GUARANTEES         *  	 		
*  *  AND NO PROMISE OF SUPPORT.                                         *  	 		
*  ***********************************************************************  	 		
*																						
* Freescale reserves the right to make changes without further notice to any			
* product herein to improve reliability, function, or design. Freescale does	 		
* not assume any  liability arising  out  of the  application or use of any 	 		
* product,  circuit, or software described herein;  neither  does it convey 	 		
* any license under its patent rights  nor the  rights of others.  Freescale	 		
* products are not designed, intended,  or authorized for use as components 	 		
* in  systems  intended  for  surgical  implant  into  the  body, or  other 	 		
* applications intended to support life, or  for any  other application  in 	 		
* which the failure of the Freescale product  could create a situation where	 		
* personal injury or death may occur. Should Buyer purchase or use Freescale	 		
* products for any such intended  or unauthorized  application, Buyer shall 	 		
* indemnify and  hold  Freescale  and its officers, employees, subsidiaries,	 		
* affiliates,  and distributors harmless against all claims costs, damages, 	  		
* and expenses, and reasonable  attorney  fees arising  out of, directly or 	 		
* indirectly,  any claim of personal injury  or death  associated with such 	 		
* unintended or unauthorized use, even if such claim alleges that  Freescale	 		
* was negligent regarding the  design  or manufacture of the part. Freescale	 		
* and the Freescale logo* are registered trademarks of Freescale Ltd.       	 		
******************************************************************************/		


/* INCLUDE FILES */
#include <hidef.h> /* for EnableInterrupts macro */
#include <MC9S08DZ60.h>
#include "bout.h"

/*-----------------------------------------------------------------------
/ Non-volatile Memory FLASH Protection Register Initialization 
/          All EEPROM and all FLASH is block protected. This prevents accidental 
/          programming or erasing of memory during brownout.         
/          Memory Security is also enabled to prevent unauthorized access to any 
/          memory space. 
/          ** In order to demonstrate security enabled, NVOPT declaration must be 
/             uncommented. Currently it is commented out so the user can see the code
/             running in the debugger. **
/------------------------------------------------------------------------*/

 //const byte NVOPT_INIT @0x0000FFBF = 0x00;  /* UNCOMMENT to enable security  */
 const byte NVPROT_INIT @0x0000FFBD = 0x00; /* All EEPROM and FLASH blk protected */


/******************************************************************************
/ Function Name	:	main
/ Notes			    :	The main function in the program - Performs the initialization
/                 of all necessary modules, enables LVD, windowed COP, and in main
/                 loop, clears COP in proper time window.                 
******************************************************************************/



void main(void) {
    word index = 0;   /* Loop index for windowed COP */
    DisableInterrupts; 
    PortsInit();
    ResetErrorHandler();
    
    /*------------------------------------------------------------------- 
    / LVD Initialization 
    /          Detects when a brownout condition is about to 
    /          happen and VDD is dropping.
    /--------------------------------------------------------------------*/
    
    SPMSC2 = 0x20;    /* LVD lvl=4.0V , LVW lvl=4.3V */
    SPMSC1 = 0x1D;    /* LVD resets, LVD enabled in STOP, BG buffer enabled */ 
    
    
    TestOptions();    /* Use PTB DIP switch to select which options to test */
    MCGInit();        /* Initialize MCG */
    PWMInit();        /* Initialize TPM for PWM */
    
    /*--------------------------------------------------------------------
    / Windowed COP Initialization  
    /          The COP will reset if timer is cleared outside of the final 
    /          25% window of the timeout period or if 0x55 then 0xAA is not 
    /          written to the SRS register in that order.  During brownout,  
    /          if code behaves inappropriately and does not properly clear the 
    /          COP during the expected time period, a COP reset will occur.
    /---------------------------------------------------------------------*/
    
    SOPT2 = 0xC0;     /* bus clk sources COP timer; use windowed COP */
    
    EnableInterrupts; /* enable interrupts - 1 cycle */


    /*------------------------------------------------------------------------
    / Windowed COP Calculations
    /          The COP is cleared in the main loop of the program. With careful
    /          cycle counting, the code can be written so that the COP is cleared
    /          in the last 25% of the COP timeout period.
    /        
    /          2^13 = 8192 bus cycles before COP timeout
    /          75% of 8192 =  6144
    /          Software can clear COP between 6144 and 8192 bus cycles
    /------------------------------------------------------------------------*/  
    for(;;) {
       while (index<300) {   /* "Dummy" Loop to clear COP at proper time. Cycle count code here */
           index++;          /* (21 bus cycles x 300) + 3 overhead + 1(cli) + 11(reset index) */ 
        }                    /* = 6312 bus cycles (btwn 6144 and 8192) */
        index = 0;           /* CodeWarrior 5.0 optimized to 11 cycles */                 
        __RESET_WATCHDOG();  /* Reset COP */
    } 
 
}


/******************************************************************************
/ Interrupt Service Routines  
******************************************************************************/
 
interrupt 3 void LVW_ISR (void) {
    
    TPM1SC_CLKSx = 0b00; /* Disable TPM (Shut down peripherals safely) */
    PTDD_PTDD6 = 0;      /* PTD6 LED ON to indicate LVW event occurred - possible brownout*/
    
    /*--------------------------------------------------------------------------
    / Voltage Recovery using ADC (use only if windowed COP disabled) - 
    /         The ADC is set up here to check if VDD has risen above 4.7 V (load
    /         on system that was pulling VDD low has been released).  ADC is 
    /         configured for continuous conversions to check Vbg and triggers an 
    /         interrupt when VDD = 4.7 V or higher. (BGVAL is compare value determined
    /         by Vbg-VDD ratio relationship.
    /
    /         Note that with windowed COP enabled, the COP will generate a Reset if
    /         interrupt service routine is more than a few cycles long.  Therefore,
    /         with ADC voltage recovery sequence, a COP reset will recover the MCU
    /         instead of the recovery sequence.  This sequence should only be used
    /         if the COP is disabled.  
    /
    /         *NOTE* 
    /         The windowed COP will also generate a reset
    /         if the MCU is in an LVW condition for a long period of time, causing
    /         LVW interrupts to continuously trigger without allowing the main loop
    /         to clear the COP in the appropriate time frame.
    /---------------------------------------------------------------------------*/
       
    ADCCFG = 0x18;           /* Async clk src, 10-bit mode, clk div 1 */
    ADCCVH = BGVAL>>8;       /* High byte of bandgap val for Vdd lvl to trigger on */
    ADCCVL = BGVAL & 0xff;   /* Low byte of bandgap val for Vdd lvl to trigger on*/
    ADCSC2 = 0x20;          /* Compare fn. enabled - triggers when input<BGVAL (Vdd > 4.7V) */
    ADCSC1 = 0x7B;          /* Continuous conv on Vbg channel with interrupts enabled */
    
        
        
    /* NOTE: 
       If there is enough time left before the LVD trip point causes a 
       reset (use a capacitor or other circuitry on power supply line to increase 
       this time period), small amounts of important data can be saved to FLASH 
       here.
       
     */
    SPMSC1_LVWACK = 1;   /* Clear LVW interrupt flag */ 
}

interrupt 4 void MCGLOL_ISR (void) {
    MCGSC_LOLS = 1;  /* Clear loss of lock flag */
    PTDD_PTDD7 = 0;  /* PTD7 LED ON to indicate LOL occurred */
}


interrupt 23 void ADC_ISR (void) {  /* Triggers after LVW event if Vdd has risen back up to 4.7 V */
    byte resulth, resultl;
    resulth = ADCRH;                /* COCO bit clearing procedure */
    resultl = ADCRL;     
    PTDD_PTDD6 = 1;                 /* PTD6 LED OFF to indicate voltage has risen above */ 
                                    /* 4.7 V (or voltage determined by BGVAL ratio)  */
    TPM1SC_CLKSx = 0b01;            /* Re-enable TPM (Re-enable peripherals ) */                                
}                                   